@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009 Yoshikazu Kuramochi
+ * Copyright (c) 2009,2010 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -49,12 +49,14 @@ import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;
 import org.eclipse.ui.application.ActionBarAdvisor;
 import org.eclipse.ui.application.IActionBarConfigurer;
 
+import ch.kuramo.javie.app.actions.AVIOutputAction;
 import ch.kuramo.javie.app.actions.NewCompositionAction;
 import ch.kuramo.javie.app.actions.NewFileItemsAction;
 import ch.kuramo.javie.app.actions.NewFolderAction;
 import ch.kuramo.javie.app.actions.NewProjectAction;
 import ch.kuramo.javie.app.actions.NewSolidColorItemAction;
 import ch.kuramo.javie.app.actions.OpenProjectAction;
+import ch.kuramo.javie.app.actions.QTMovieOutputAction;
 import ch.kuramo.javie.app.actions.RedoAction;
 import ch.kuramo.javie.app.actions.SaveAction;
 import ch.kuramo.javie.app.actions.SaveAsAction;
@@ -92,6 +94,10 @@ public class ApplicationActionBarAdvisor extends ActionBarAdvisor {
 
 	private NewFileItemsAction newFileItemsAction;
 
+	private RetargetAction aviOutputAction;
+
+	private RetargetAction qtMovieOutputAction;
+
 	private RetargetAction sequenceOutputAction;
 
 	private RetargetAction waveOutputAction;
@@ -116,6 +122,8 @@ public class ApplicationActionBarAdvisor extends ActionBarAdvisor {
 
 	private IWorkbenchAction aboutAction;
 
+	private MenuManager recentProjectsMenu;
+
 	@Inject
 	private EffectRegistry effectRegistry;
 
@@ -157,6 +165,18 @@ public class ApplicationActionBarAdvisor extends ActionBarAdvisor {
 		newFileItemsAction = new NewFileItemsAction(window);
 		register(newFileItemsAction);
 
+		if (AVIOutputAction.isAvailable()) {
+			aviOutputAction = new RetargetAction(CommandIds.AVI_OUTPUT, "AVI...");
+			register(aviOutputAction);
+			window.getPartService().addPartListener(aviOutputAction);
+		}
+
+		if (QTMovieOutputAction.isAvailable()) {
+			qtMovieOutputAction = new RetargetAction(CommandIds.QTMOVIE_OUTPUT, "QuickTime ムービー...");
+			register(qtMovieOutputAction);
+			window.getPartService().addPartListener(qtMovieOutputAction);
+		}
+
 		sequenceOutputAction = new RetargetAction(CommandIds.SEQUENCE_OUTPUT, "イメージシーケンス...");
 		register(sequenceOutputAction);
 		window.getPartService().addPartListener(sequenceOutputAction);
@@ -204,6 +224,8 @@ public class ApplicationActionBarAdvisor extends ActionBarAdvisor {
 		register(aboutAction);
 
 
+		recentProjectsMenu = RecentProjects.createMenuManager(window);
+
 		makeEffectActions(window);
 	}
 
@@ -266,6 +288,7 @@ public class ApplicationActionBarAdvisor extends ActionBarAdvisor {
 		// File
 		fileMenu.add(fileNewMenu);
 		fileMenu.add(openProjectAction);
+		fileMenu.add(recentProjectsMenu);
 		fileMenu.add(new Separator());
 		fileMenu.add(saveAction);
 		fileMenu.add(saveAsAction);
@@ -288,6 +311,8 @@ public class ApplicationActionBarAdvisor extends ActionBarAdvisor {
 		inputMenu.add(newFileItemsAction);
 
 		// File/Output
+		if (aviOutputAction != null) outputMenu.add(aviOutputAction);
+		if (qtMovieOutputAction != null) outputMenu.add(qtMovieOutputAction);
 		outputMenu.add(sequenceOutputAction);
 		outputMenu.add(waveOutputAction);
 
