@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009 Yoshikazu Kuramochi
+ * Copyright (c) 2009,2010 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -24,6 +24,7 @@
 package ch.kuramo.javie.app.player;
 
 import java.nio.ByteBuffer;
+import java.nio.ByteOrder;
 import java.util.Arrays;
 import java.util.concurrent.atomic.AtomicReference;
 
@@ -71,6 +72,8 @@ public class AudioThread extends Thread {
 
 	private volatile boolean _finished;
 
+	private volatile boolean _muted;
+
 	@Inject
 	private AudioRenderContext _arContext;
 
@@ -90,7 +93,7 @@ public class AudioThread extends Thread {
 			throw new IllegalArgumentException("no audio is available");
 		}
 
-		AudioFormat lineFormat = new AudioFormat(48000, 16, 2, true, true);
+		AudioFormat lineFormat = new AudioFormat(48000, 16, 2, true, ByteOrder.nativeOrder().equals(ByteOrder.BIG_ENDIAN));
 		DataLine.Info lineInfo = new DataLine.Info(SourceDataLine.class, lineFormat);
 		_line = (SourceDataLine) AudioSystem.getLine(lineInfo);
 		_line.open(lineFormat, lineFormat.getFrameSize() * (int) (lineFormat.getFrameRate() / 10));	// 0.1秒分のバッファサイズ
@@ -106,6 +109,14 @@ public class AudioThread extends Thread {
 		return _timeKeeper;
 	}
 
+	public boolean isMuted() {
+		return _muted;
+	}
+
+	public void setMuted(boolean muted) {
+		_muted = muted;
+	}
+
 	public void run() {
 		try {
 			MediaInput input = _mediaItem.getMediaInput();
@@ -142,32 +153,34 @@ public class AudioThread extends Thread {
 					continue;
 				}
 
-				_arContext.reset();
-				_arContext.setAudioMode(audioMode);
-				_arContext.setTime(time);
-				_arContext.setFrameCount(frameCount);
-
 				AudioBuffer ab = null;
 
-				if (_mediaItem instanceof CompositionItem) {
-					// TODO 上の input.getDuration() も厳密にはロックする必要がある。
-					PlayerLock.readLock().lock();
-					try {
-						// TODO prepareExpressionはプロジェクトに構造的な変更があった場合のみ行えばよい。
-						Composition comp = ((CompositionItem) _mediaItem).getComposition();
-						comp.prepareExpression(_arContext.createInitialExpressionScope());
-	
-						ab = input.getAudioChunk();
-					} catch (Exception e) {
-						_logger.error("error getting audio chunk", e);
-					} finally {
-						PlayerLock.readLock().unlock();
-					}
-				} else {
-					try {
-						ab = input.getAudioChunk();
-					} catch (Exception e) {
-						_logger.error("error getting audio chunk", e);
+				if (!_muted) {
+					_arContext.reset();
+					_arContext.setAudioMode(audioMode);
+					_arContext.setTime(time);
+					_arContext.setFrameCount(frameCount);
+
+					if (_mediaItem instanceof CompositionItem) {
+						// TODO 上の input.getDuration() も厳密にはロックする必要がある。
+						PlayerLock.readLock().lock();
+						try {
+							// TODO prepareExpressionはプロジェクトに構造的な変更があった場合のみ行えばよい。
+							Composition comp = ((CompositionItem) _mediaItem).getComposition();
+							comp.prepareExpression(_arContext.createInitialExpressionScope());
+
+							ab = input.getAudioChunk();
+						} catch (Exception e) {
+							_logger.error("error getting audio chunk", e);
+						} finally {
+							PlayerLock.readLock().unlock();
+						}
+					} else {
+						try {
+							ab = input.getAudioChunk();
+						} catch (Exception e) {
+							_logger.error("error getting audio chunk", e);
+						}
 					}
 				}
 
@@ -248,13 +261,14 @@ public class AudioThread extends Thread {
 
 		switch (audioMode.dataType) {
 			case SHORT:
-				ByteBuffer.wrap(lineBuf).asShortBuffer().put((short[]) ab.getData(), 0, dataLen);
+				ByteBuffer.wrap(lineBuf).order(ByteOrder.nativeOrder())
+						.asShortBuffer().put((short[]) ab.getData(), 0, dataLen);
 				break;
 			case INT: {
 				int[] data = (int[]) ab.getData();
 				for (int i = 0; i < dataLen; ++i) {
-					lineBuf[i*2  ] = (byte) ((data[i] >>> 24) & 0xff);
-					lineBuf[i*2+1] = (byte) ((data[i] >>> 16) & 0xff);
+					lineBuf[i*2  ] = (byte) ((data[i] >>> 16) & 0xff);
+					lineBuf[i*2+1] = (byte) ((data[i] >>> 24) & 0xff);
 				}
 				break;
 			}
@@ -262,8 +276,8 @@ public class AudioThread extends Thread {
 				float[] data = (float[]) ab.getData();
 				for (int i = 0; i < dataLen; ++i) {
 					short shortVal = (short) (Math.min(Math.max(data[i], -1.0), 1.0) * Short.MAX_VALUE);
-					lineBuf[i*2  ] = (byte) ((shortVal >>> 8) & 0xff);
-					lineBuf[i*2+1] = (byte) ((shortVal      ) & 0xff);
+					lineBuf[i*2  ] = (byte) ((shortVal      ) & 0xff);
+					lineBuf[i*2+1] = (byte) ((shortVal >>> 8) & 0xff);
 				}
 				break;
 			}
@@ -274,11 +288,11 @@ public class AudioThread extends Thread {
 
 		if (audioMode.sampleRate == 96000) {
 			for (int i = 0, n = dataLen/2; i < n; ++i) {
-				short s1 = (short) (((lineBuf[i*2*2  ] & 0xff) << 8) | (lineBuf[i*2*2+1] & 0xff));
-				short s2 = (short) (((lineBuf[i*2*2+2] & 0xff) << 8) | (lineBuf[i*2*2+3] & 0xff));
+				short s1 = (short) ((lineBuf[i*2*2  ] & 0xff) | ((lineBuf[i*2*2+1] & 0xff) << 8));
+				short s2 = (short) ((lineBuf[i*2*2+2] & 0xff) | ((lineBuf[i*2*2+3] & 0xff) << 8));
 				short s = (short) ((s1 + s2) / 2);
-				lineBuf[i*2  ] = (byte) ((s >>> 8) & 0xff);
-				lineBuf[i*2+1] = (byte) ((s      ) & 0xff);
+				lineBuf[i*2  ] = (byte) ((s      ) & 0xff);
+				lineBuf[i*2+1] = (byte) ((s >>> 8) & 0xff);
 			}
 			lineDataLen /= 2;
 		}
