@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009 Yoshikazu Kuramochi
+ * Copyright (c) 2009,2010 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -25,9 +25,13 @@ package ch.kuramo.javie.app.player;
 
 import java.awt.Dimension;
 import java.awt.EventQueue;
+import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Iterator;
+import java.util.LinkedHashMap;
+import java.util.List;
 import java.util.Queue;
+import java.util.Map.Entry;
 import java.util.concurrent.ConcurrentLinkedQueue;
 import java.util.concurrent.atomic.AtomicReference;
 
@@ -44,6 +48,7 @@ import org.eclipse.swt.SWT;
 import org.eclipse.swt.custom.ScrolledComposite;
 import org.eclipse.swt.events.ControlAdapter;
 import org.eclipse.swt.events.ControlEvent;
+import org.eclipse.swt.events.ControlListener;
 import org.eclipse.swt.events.SelectionAdapter;
 import org.eclipse.swt.events.SelectionEvent;
 import org.eclipse.swt.graphics.Rectangle;
@@ -53,6 +58,7 @@ import org.eclipse.swt.widgets.ScrollBar;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import ch.kuramo.javie.api.Color;
 import ch.kuramo.javie.api.ColorMode;
 import ch.kuramo.javie.api.RenderResolution;
 import ch.kuramo.javie.api.Size2i;
@@ -65,10 +71,13 @@ import ch.kuramo.javie.core.CompositionItem;
 import ch.kuramo.javie.core.FrameDuration;
 import ch.kuramo.javie.core.MediaInput;
 import ch.kuramo.javie.core.MediaItem;
+import ch.kuramo.javie.core.SolidColorItem;
+import ch.kuramo.javie.core.TimeCode;
 import ch.kuramo.javie.core.VideoBuffer;
 import ch.kuramo.javie.core.services.VideoRenderContext;
 
 import com.google.inject.Inject;
+import com.sun.opengl.util.GLUT;
 
 public class VideoCanvas extends Thread implements GLEventListener {
 
@@ -93,7 +102,9 @@ public class VideoCanvas extends Thread implements GLEventListener {
 
 	private final GLU _glu = new GLU();
 
-	private final AtomicReference<VideoBuffer> _videoBufferRef = new AtomicReference<VideoBuffer>();
+	private final GLUT _glut = new GLUT();
+
+	private final AtomicReference<Object[]> _videoBufferAndTimeRef = new AtomicReference<Object[]>();
 
 	private final Queue<VideoBuffer> _oldVideoBuffers = new ConcurrentLinkedQueue<VideoBuffer>();
 
@@ -101,7 +112,9 @@ public class VideoCanvas extends Thread implements GLEventListener {
 
 	private volatile RenderResolution _resolution = RenderResolution.FULL;
 
-	private double _zoom = Double.NaN;
+	private double _zoom;
+
+	private volatile boolean _showInfo;
 
 	private Size2i _canvasSize;
 
@@ -111,6 +124,8 @@ public class VideoCanvas extends Thread implements GLEventListener {
 
 	private int _scrollAreaHeight;
 
+	private ControlListener _resizeListener;
+
 	private volatile int[] _viewport = new int[4];
 
 	private final Object _monitor = new Object();
@@ -127,6 +142,10 @@ public class VideoCanvas extends Thread implements GLEventListener {
 
 	private final ListenerList _playerThreadListeners = new ListenerList();
 
+	private final MovingAverage _delayAverage = new MovingAverage(30);
+
+	private final MovingAverage _frameDurationAvg = new MovingAverage(30);
+
 	@Inject
 	private VideoRenderContext _vrContext;
 
@@ -155,8 +174,10 @@ public class VideoCanvas extends Thread implements GLEventListener {
 		_canvas.addGLEventListener(this);
 
 		if (COCOA) {
-			listenToScrollAndResize();
+			cocoaListenToScrollAndResize();
 			_scrollAreaHeight = _scrolled.getClientArea().height;
+		} else {
+			listenToResize();
 		}
 
 		setZoom(1.0);
@@ -169,13 +190,18 @@ public class VideoCanvas extends Thread implements GLEventListener {
 	private void cleanup() {
 		_canvas.removeGLEventListener(this);
 
+		if (_resizeListener != null) {
+			_scrolled.removeControlListener(_resizeListener);
+			_resizeListener = null;
+		}
+
 		if (_glCanvasRecord != null) {
 			GLCanvasFactory.getFactory().releaseGLCanvas(_glCanvasRecord);
 			_glCanvasRecord = null;
 		}
 	}
 
-	private void listenToScrollAndResize() {
+	private void cocoaListenToScrollAndResize() {
 		final ScrollBar hBar = _scrolled.getHorizontalBar();
 		final ScrollBar vBar = _scrolled.getVerticalBar();
 
@@ -201,7 +227,7 @@ public class VideoCanvas extends Thread implements GLEventListener {
 
 		_scrolled.addControlListener(new ControlAdapter() {
 			public void controlResized(ControlEvent e) {
-				if (Double.isNaN(_zoom)) {
+				if (_zoom == 0) {
 					fitCanvasToScrollArea();
 				}
 
@@ -225,19 +251,51 @@ public class VideoCanvas extends Thread implements GLEventListener {
 		});
 	}
 
+	private void listenToResize() {
+		_resizeListener = new ControlAdapter() {
+			public void controlResized(ControlEvent e) {
+				if (_zoom == 0) {
+					fitCanvasToScrollArea();
+				}
+
+				e.display.asyncExec(new Runnable() {
+					public void run() {
+						if (!_scrolled.isDisposed()) {
+							_glCanvasRecord.awtFrame.repaint();
+						}
+					}
+				});
+			}
+		};
+		_scrolled.addControlListener(_resizeListener);
+	}
+
 	public void setFrameDuration(Time frameDuration) {
 		_frameDuration = frameDuration;
 	}
 
+	public RenderResolution getResolution() {
+		return _resolution;
+	}
+
 	public void setResolution(RenderResolution resolution) {
-		_resolution = resolution;
+		if ((resolution != null && !resolution.equals(_resolution))
+				|| (resolution == null && _resolution != null)) {
+
+			_resolution = resolution;
+			forceRender();
+		}
+	}
+
+	public double getZoom() {
+		return _zoom;
 	}
 
 	public void setZoom(double zoom) {
 		if (_zoom != zoom) {
 			_zoom = zoom;
 
-			if (Double.isNaN(zoom)) {
+			if (zoom == 0) {
 				_scrolled.setMinSize(1, 1);
 				fitCanvasToScrollArea();
 
@@ -252,19 +310,32 @@ public class VideoCanvas extends Thread implements GLEventListener {
 		}
 	}
 
+	public boolean isShowInfo() {
+		return _showInfo;
+	}
+
+	public void setShowInfo(boolean showInfo) {
+		if (showInfo != _showInfo) {
+			_showInfo = showInfo;
+			_canvas.repaint();
+		}
+	}
+
 	private void fitCanvasToScrollArea() {
 		Rectangle scrollArea = _scrolled.getClientArea();
 		VideoBounds videoBounds = _mediaInput.getVideoFrameBounds();
 
 		int w, h;
-		double scrollAspect = (double) scrollArea.width / scrollArea.height;
+		int availableWidth = scrollArea.width - 10;
+		int availableHeight = scrollArea.height - 10;
+		double scrollAspect = (double) availableWidth / availableHeight;
 		double videoAspect = (double) videoBounds.width / videoBounds.height;
 		if (scrollAspect > videoAspect) {
-			w = (int) (scrollArea.height * videoAspect);
-			h = scrollArea.height;
+			w = (int) (availableHeight * videoAspect);
+			h = availableHeight;
 		} else {
-			w = scrollArea.width;
-			h = (int) (scrollArea.width / videoAspect);
+			w = availableWidth;
+			h = (int) (availableWidth / videoAspect);
 		}
 
 		setCanvasSize(w, h);
@@ -332,18 +403,20 @@ public class VideoCanvas extends Thread implements GLEventListener {
 
 		_displaying = true;
 
-		VideoBuffer vb = null;
+		Object[] vbAndTime = null;
 		try {
 			GL gl = drawable.getGL();
 
 			// 背景を黒以外にする場合に必要。initメソッド内のコメント参照。
-			// gl.glClear(GL.GL_COLOR_BUFFER_BIT);
+			//gl.glClear(GL.GL_COLOR_BUFFER_BIT);
 
-			vb = _videoBufferRef.getAndSet(null);
-			if (vb == null) {
+			vbAndTime = _videoBufferAndTimeRef.getAndSet(null);
+			if (vbAndTime == null) {
 				return;
 			}
 
+			VideoBuffer vb = (VideoBuffer) vbAndTime[0];
+
 			int[] vp = _viewport;
 			gl.glViewport(vp[0], vp[1], vp[2], vp[3]);
 
@@ -364,19 +437,93 @@ public class VideoCanvas extends Thread implements GLEventListener {
 			gl.glTexCoord2f(w, h); gl.glVertex2f(w, h);
 			gl.glTexCoord2f(0, h); gl.glVertex2f(0, h);
 			gl.glEnd();
-			gl.glFlush();
 
 			gl.glBindTexture(GL.GL_TEXTURE_RECTANGLE_EXT, 0);
 
+			if (_showInfo) {
+				gl.glPushAttrib(GL.GL_ENABLE_BIT | GL.GL_CURRENT_BIT | GL.GL_COLOR_BUFFER_BIT);
+
+				gl.glLoadIdentity();
+				_glu.gluOrtho2D(0, vp[2], vp[3], 0);
+
+				drawInfo(gl, (Time) vbAndTime[1]);
+
+				gl.glPopAttrib();
+			}
+
+			gl.glFinish();
+
 		} finally {
-			if (vb != null && !_videoBufferRef.compareAndSet(null, vb)) {
-				_oldVideoBuffers.add(vb);
+			if (vbAndTime != null && !_videoBufferAndTimeRef.compareAndSet(null, vbAndTime)) {
+				_oldVideoBuffers.add((VideoBuffer) vbAndTime[0]);
 			}
 
 			_displaying = false;
 		}
 	}
 
+	private void drawInfo(GL gl, Time time) {
+		List<String> info = new ArrayList<String>();
+
+		VideoBounds bounds = _mediaInput.getVideoFrameBounds();
+		Time frameDuration = _mediaInput.getVideoFrameDuration();
+
+		if (_mediaItem instanceof CompositionItem) {
+			String bpc;
+			switch (((CompositionItem) _mediaItem).getComposition().getColorMode()) {
+				case RGBA8: bpc = "8 bpc"; break;
+				case RGBA16: bpc = "16 bpc"; break;
+				case RGBA16_FLOAT: bpc = "16 bpc float"; break;
+				case RGBA32_FLOAT: bpc = "32 bpc float"; break;
+				default: bpc = "unknown bpc"; break;
+			}
+			info.add(String.format("Composition: %d x %d, %s", bounds.width, bounds.height, bpc));
+
+		} else if (_mediaItem instanceof SolidColorItem) {
+			Color color = ((SolidColorItem) _mediaItem).getColor();
+			info.add(String.format("Solid Color: %d x %d, #%02x%02x%02x", bounds.width, bounds.height,
+					(int)(color.r*255), (int)(color.g*255), (int)(color.b*255)));
+
+		} else if (_mediaInput.getVideoFrameDuration() != null) {
+			info.add(String.format("Video: %d x %d", bounds.width, bounds.height));
+
+		} else {
+			info.add(String.format("Image: %d x %d", bounds.width, bounds.height));
+		}
+
+		if (frameDuration != null) {
+			Time duration = _mediaInput.getDuration();
+			info.add(String.format("Frame: %d / %d", time.toFrameNumber(frameDuration), duration.toFrameNumber(frameDuration)));
+			info.add(String.format("Time: %s / %s", TimeCode.toTimeCode(time, frameDuration), TimeCode.toTimeCode(duration, frameDuration)));
+			info.add(String.format("FPS: %.2f / %.2f", 1/_frameDurationAvg.getSampledAverage(), 1/frameDuration.toSecond()));
+		}
+
+		int infoWidth = 0;
+		int infoHeight = info.size() * 12;
+		for (String s : info) {
+			infoWidth = Math.max(infoWidth, _glut.glutBitmapLength(GLUT.BITMAP_HELVETICA_10, s));
+		}
+		infoWidth += 2;
+
+		gl.glEnable(GL.GL_BLEND);
+		gl.glBlendFunc(GL.GL_SRC_ALPHA, GL.GL_ONE_MINUS_SRC_ALPHA);
+		gl.glColor4f(0, 0, 0, 0.4f);
+		gl.glBegin(GL.GL_QUADS);
+		gl.glVertex2f(10, 10);
+		gl.glVertex2f(10+infoWidth, 10);
+		gl.glVertex2f(10+infoWidth, 10+infoHeight);
+		gl.glVertex2f(10, 10+infoHeight);
+		gl.glEnd();
+
+		gl.glDisable(GL.GL_BLEND);
+		gl.glColor4f(0, 1, 0, 1);
+		int i = 0;
+		for (String s : info) {
+			gl.glRasterPos2f(11, 20+(i++)*12);
+			_glut.glutBitmapString(GLUT.BITMAP_HELVETICA_10, s);
+		}
+	}
+
 	public void reshape(GLAutoDrawable drawable, int x, int y, int width, int height) {
 		// nothing to do
 	}
@@ -392,11 +539,15 @@ public class VideoCanvas extends Thread implements GLEventListener {
 			GLContext glContext = GLContext.getCurrent();
 			glContext.setGL(new DebugGL(glContext.getGL()));
 
+			GL gl = _vrContext.getGL();
+
 			Time duration = _mediaInput.getDuration();
 			if (duration == null) {
-				Time time = Time.fromFrameNumber(0, FrameDuration.FPS_29_97);
+				RenderResolution resolution = _resolution;
+
+				Time time = TIME0;
 				_vrContext.reset();
-				_vrContext.setRenderResolution(_resolution);
+				_vrContext.setRenderResolution(resolution);
 				_vrContext.setTime(time);
 				_vrContext.setColorMode(ColorMode.RGBA8);
 
@@ -411,16 +562,20 @@ public class VideoCanvas extends Thread implements GLEventListener {
 				}
 
 				if (vb != null) {
-					_vrContext.getGL().glFlush();
-					_videoBufferRef.getAndSet(vb);
+					gl.glFinish();
+
+					setTexParametersForDisplay(vb, resolution, gl);
+
+					_videoBufferAndTimeRef.getAndSet(new Object[] { vb, TIME0 });
 					_canvas.repaint();
 					fireThreadRender(time);
 				}
 				fireThreadEndOfDuration(time);
 
-				while (waitForRendering() != null) {
-					_forceRender = false;
-					fireThreadEndOfDuration(time);
+				synchronized (_monitor) {
+					while (!_finished) {
+						try { _monitor.wait(); } catch (InterruptedException e) { }
+					}
 				}
 				return;
 			}
@@ -431,6 +586,7 @@ public class VideoCanvas extends Thread implements GLEventListener {
 			Time[] timeAndBase;
 			Time prevBase = _timeKeeper.getTimeAndBase()[1];
 			Time delay = TIME0;
+			RenderResolution prevResolution = null;
 
 			while ((timeAndBase = waitForRendering()) != null) {
 				boolean forceRender = !_playing && _forceRender;
@@ -475,8 +631,19 @@ public class VideoCanvas extends Thread implements GLEventListener {
 					_timeKeeper.setTime(frameTime);
 				}
 
+				RenderResolution resolution = _resolution;
+				if (resolution != null) {
+					prevResolution = null;
+				} else if (forceRender) {
+					resolution = RenderResolution.FULL;
+					prevResolution = null;
+				} else {
+					resolution = autoResolution(prevResolution, frameDuration.toSecond()*1.5);
+					prevResolution = resolution;
+				}
+
 				_vrContext.reset();
-				_vrContext.setRenderResolution(_resolution);
+				_vrContext.setRenderResolution(resolution);
 				_vrContext.setTime(frameTime);
 				_vrContext.setColorMode(ColorMode.RGBA8);
 
@@ -511,7 +678,7 @@ public class VideoCanvas extends Thread implements GLEventListener {
 				}
 
 				if (vb != null) {
-					_vrContext.getGL().glFlush();
+					gl.glFinish();
 
 					delay = calcDelay(timeAndBase);
 
@@ -521,10 +688,12 @@ public class VideoCanvas extends Thread implements GLEventListener {
 						delay = TIME0;
 
 					} else {
-						VideoBuffer old = _videoBufferRef.getAndSet(vb);
+						setTexParametersForDisplay(vb, resolution, gl);
+
+						Object[] old = _videoBufferAndTimeRef.getAndSet(new Object[] { vb, frameTime });
 						_canvas.repaint();
 						if (old != null) {
-							old.dispose();
+							((VideoBuffer) old[0]).dispose();
 						}
 
 						fireThreadRender(frameTime);
@@ -534,6 +703,14 @@ public class VideoCanvas extends Thread implements GLEventListener {
 						}
 
 						prevFrame = frameNumber;
+
+						if (forceRender) {
+							_delayAverage.reset();
+							_frameDurationAvg.reset();
+						} else {
+							_delayAverage.add(delay.toSecond());
+							_frameDurationAvg.add((delay.before(frameDuration) ? frameDuration : delay).toSecond());
+						}
 					}
 				}
 
@@ -549,9 +726,9 @@ public class VideoCanvas extends Thread implements GLEventListener {
 				sleep(FrameDuration.FPS_29_97);
 			}
 
-			VideoBuffer vb = _videoBufferRef.getAndSet(null);
-			if (vb != null) {
-				vb.dispose();
+			Object[] vbAndTime = _videoBufferAndTimeRef.getAndSet(null);
+			if (vbAndTime != null) {
+				((VideoBuffer) vbAndTime[0]).dispose();
 			}
 			disposeOldVideoBuffers();
 
@@ -596,6 +773,54 @@ public class VideoCanvas extends Thread implements GLEventListener {
 		cleanup();
 	}
 
+	private void setTexParametersForDisplay(VideoBuffer vb, RenderResolution resolution, GL gl) {
+		gl.glBindTexture(GL.GL_TEXTURE_RECTANGLE_EXT, vb.getTexture());
+
+		// フル解像度のときは、拡大したときにテクスチャの補間が効かないようにする。
+		// フル解像度でないときは補間を有効にするが、補間によってエッジ部分が
+		// 黒とブレンドされないようにラッピングモードを GL_CLAMP_TO_EDGE に設定する。
+
+		if (RenderResolution.FULL.equals(resolution)) {
+			// GL_TEXTURE_MAG_FILTER だけ設定すればいいはずだが、
+			// ATIではなぜかダメなようなので MIN/MAG 両方を設定している。
+			gl.glTexParameteri(GL.GL_TEXTURE_RECTANGLE_EXT, GL.GL_TEXTURE_MIN_FILTER, GL.GL_NEAREST);
+			gl.glTexParameteri(GL.GL_TEXTURE_RECTANGLE_EXT, GL.GL_TEXTURE_MAG_FILTER, GL.GL_NEAREST);
+
+		} else {
+			gl.glTexParameteri(GL.GL_TEXTURE_RECTANGLE_EXT, GL.GL_TEXTURE_MIN_FILTER, GL.GL_LINEAR);
+			gl.glTexParameteri(GL.GL_TEXTURE_RECTANGLE_EXT, GL.GL_TEXTURE_MAG_FILTER, GL.GL_LINEAR);
+			gl.glTexParameteri(GL.GL_TEXTURE_RECTANGLE_EXT, GL.GL_TEXTURE_WRAP_S, GL.GL_CLAMP_TO_EDGE);
+			gl.glTexParameteri(GL.GL_TEXTURE_RECTANGLE_EXT, GL.GL_TEXTURE_WRAP_T, GL.GL_CLAMP_TO_EDGE);
+		}
+
+		gl.glBindTexture(GL.GL_TEXTURE_RECTANGLE_EXT, 0);
+	}
+
+	private RenderResolution autoResolution(RenderResolution prevResolution, double avgThreshold) {
+		if (prevResolution == null) {
+			return RenderResolution.FULL;
+
+		// avgThreshold 以下のときは現在の解像度を維持する。
+		// MovingAverage.getAverage() は要素数が少ないとNaNを返すので不等号の向きに注意。
+		} else if (_delayAverage.getAverage() > avgThreshold) {
+
+			if (prevResolution.equals(RenderResolution.FULL)) {
+				_delayAverage.reset();
+				return RenderResolution.HALF;
+
+			} else if (prevResolution.equals(RenderResolution.HALF)) {
+				_delayAverage.reset();
+				return RenderResolution.ONETHIRD;
+
+			} else {
+				return RenderResolution.QUARTER;
+			}
+
+		} else {
+			return prevResolution;
+		}
+	}
+
 	private Time[] waitForRendering() {
 		if (!_playing && !_forceRender) {
 			synchronized (_monitor) {
@@ -682,3 +907,71 @@ public class VideoCanvas extends Thread implements GLEventListener {
 	}
 
 }
+
+class MovingAverage {
+
+	@SuppressWarnings("serial")
+	private final LinkedHashMap<Object, Double> map = new LinkedHashMap<Object, Double>() {
+
+		protected boolean removeEldestEntry(Entry<Object, Double> eldest) {
+			if (size() > numEntries) {
+				sum -= eldest.getValue();
+				return true;
+			}
+			return false;
+		}
+	};
+
+
+	private final int numEntries;
+
+	private final int sampleInterval;
+
+	private double sum;
+
+	private volatile double average;
+
+	private int counter;
+
+	private volatile double sampledAverage;
+
+
+	MovingAverage(int numEntries, int sampleInterval) {
+		this.numEntries = numEntries;
+		this.sampleInterval = sampleInterval;
+	}
+
+	MovingAverage(int numEntries) {
+		this(numEntries, 10);
+	}
+
+	void add(double value) {
+		map.put(new Object(), value);
+
+		sum += value;
+
+		int size = map.size();
+		average = (size < numEntries) ? Double.NaN : sum/size;
+
+		if ((counter++ % sampleInterval) == 0) {
+			sampledAverage = average;
+		}
+	}
+
+	double getAverage() {
+		return average;
+	}
+
+	double getSampledAverage() {
+		return sampledAverage;
+	}
+
+	void reset() {
+		map.clear();
+		sum = 0;
+		average = Double.NaN;
+		counter = 0;
+		sampledAverage = Double.NaN;
+	}
+
+}
