@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009 Yoshikazu Kuramochi
+ * Copyright (c) 2009,2010 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -91,11 +91,13 @@ import org.eclipse.swt.widgets.TreeColumn;
 import org.eclipse.swt.widgets.TreeItem;
 import org.eclipse.ui.IActionBars;
 import org.eclipse.ui.IPartListener;
+import org.eclipse.ui.IViewPart;
 import org.eclipse.ui.IViewSite;
 import org.eclipse.ui.IWorkbenchActionConstants;
 import org.eclipse.ui.IWorkbenchPage;
 import org.eclipse.ui.IWorkbenchPart;
 import org.eclipse.ui.IWorkbenchWindow;
+import org.eclipse.ui.PartInitException;
 import org.eclipse.ui.part.ViewPart;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -107,8 +109,10 @@ import ch.kuramo.javie.app.InjectorHolder;
 import ch.kuramo.javie.app.IntervalTask;
 import ch.kuramo.javie.app.PropertyUtil;
 import ch.kuramo.javie.app.UIUtil;
+import ch.kuramo.javie.app.actions.AVIOutputAction;
 import ch.kuramo.javie.app.actions.AddEffectAction;
 import ch.kuramo.javie.app.actions.AddExpressionsAction;
+import ch.kuramo.javie.app.actions.QTMovieOutputAction;
 import ch.kuramo.javie.app.actions.RemoveKeyframesAction;
 import ch.kuramo.javie.app.actions.SequenceOutputAction;
 import ch.kuramo.javie.app.actions.TimeRemapAction;
@@ -164,10 +168,12 @@ public class LayerCompositionView extends ViewPart
 
 	public static final int NAME_COL		= 0;
 	public static final int VALUE_COL		= 1;
-	public static final int PARENT_COL		= 2;
-	public static final int SHOWHIDE_COL	= 3;
-	public static final int LABEL_COL		= 4;
-	public static final int TIMELINE_COL	= 5;
+	public static final int MODE_COL		= 2;
+	public static final int TRACKMATTE_COL	= 3;
+	public static final int PARENT_COL		= 4;
+	public static final int SHOWHIDE_COL	= 5;
+	public static final int LABEL_COL		= 6;
+	public static final int TIMELINE_COL	= 7;
 
 	public static final String PROJECT_MANAGER = "PROJECT_MANAGER";
 	public static final String LAYER_COMPOSITION = "LAYER_COMPOSITION";
@@ -251,6 +257,11 @@ public class LayerCompositionView extends ViewPart
 	}
 
 	public void createPartControl(final Composite parent) {
+		ProjectManager pm = ProjectManager.forWorkbenchWindow(getSite().getWorkbenchWindow());
+		if (pm == null) {
+			throw new IllegalStateException("No ProjectManager exists for this window.");
+		}
+
 		parent.setLayout(new FormLayout());
 
 		_topLeftSpacer = new Composite(parent, SWT.NONE);
@@ -279,13 +290,7 @@ public class LayerCompositionView extends ViewPart
 			}
 		};
 
-
-		ProjectManager pm = ProjectManager.forWorkbenchWindow(getSite().getWorkbenchWindow());
-		if (pm == null) {
-			setPartName("");
-		} else if (!setProjectManager(pm)) {
-			throw new IllegalStateException("no such CompositionItem found: " + getItemId());
-		}
+		setProjectManager(pm);
 
 		ProjectEventHub.getHub().addProjectListener(this);
 	}
@@ -306,10 +311,10 @@ public class LayerCompositionView extends ViewPart
 		return getViewSite().getSecondaryId();
 	}
 
-	private boolean setProjectManager(ProjectManager pm) {
+	private void setProjectManager(ProjectManager pm) {
 		_compItem = pm.getProject().getItem(getItemId());
 		if (_compItem == null) {
-			return false;
+			throw new IllegalStateException("no such CompositionItem found: " + getItemId());
 		}
 
 		_projectManager = pm;
@@ -329,7 +334,6 @@ public class LayerCompositionView extends ViewPart
 
 		collectAnimatableValues();
 		connectToPlayer();
-		return true;
 	}
 
 	public void handleProjectEvent(ProjectEvent event) {
@@ -349,17 +353,6 @@ public class LayerCompositionView extends ViewPart
 		}
 
 		switch (event.type) {
-			case PROJECT_INITIALIZE:
-				if (!setProjectManager(pm)) {
-					_logger.warn("no such CompositionItem found: " + getItemId());
-					UIUtil.hideView(getSite().getPage(), this);
-				}
-				break;
-
-			case PROJECT_DISPOSE:
-				UIUtil.hideView(getSite().getPage(), this);
-				break;
-
 			case ITEM_NAME_CHANGE:
 				handleItemNameChange(event);
 				break;
@@ -454,12 +447,14 @@ public class LayerCompositionView extends ViewPart
 		final DragSourceEffect dragEffect = new DragSourceEffect(tree);
 		tree.setData("DEFAULT_DRAG_SOURCE_EFFECT", dragEffect);
 
-		createViewerColumn(NAME_COL,     200, null, "ソース名");
-		createViewerColumn(VALUE_COL,    150, null, "");
-		createViewerColumn(PARENT_COL,   100, null, "親");
-		createViewerColumn(SHOWHIDE_COL,  60, null, "");
-		createViewerColumn(LABEL_COL,     40, null, "");
-		createViewerColumn(TIMELINE_COL, 300, null, "");
+		createViewerColumn(NAME_COL,      200, null, "ソース名");
+		createViewerColumn(VALUE_COL,     150, null, "");
+		createViewerColumn(MODE_COL,       50, null, "モード");
+		createViewerColumn(TRACKMATTE_COL, 50, null, "ﾄﾗｯｸﾏｯﾄ");
+		createViewerColumn(PARENT_COL,    100, null, "親");
+		createViewerColumn(SHOWHIDE_COL,   60, null, "");
+		createViewerColumn(LABEL_COL,      40, null, "");
+		createViewerColumn(TIMELINE_COL,  300, null, "");
 
 
 		ControlAdapter columnWidthAdjuster = new ControlAdapter() {
@@ -638,6 +633,12 @@ public class LayerCompositionView extends ViewPart
 		actionBars.setGlobalActionHandler(CommandIds.NEW_CAMERA_LAYER, _newCameraLayerAction);
 		actionBars.setGlobalActionHandler(CommandIds.NEW_NULL_LAYER, _newNullLayerAction);
 		actionBars.setGlobalActionHandler(CommandIds.NEW_TEXT_LAYER, _newTextLayerAction);
+		if (AVIOutputAction.isAvailable()) {
+			actionBars.setGlobalActionHandler(CommandIds.AVI_OUTPUT, new AVIOutputAction(this));
+		}
+		if (QTMovieOutputAction.isAvailable()) {
+			actionBars.setGlobalActionHandler(CommandIds.QTMOVIE_OUTPUT, new QTMovieOutputAction(this));
+		}
 		actionBars.setGlobalActionHandler(CommandIds.SEQUENCE_OUTPUT, new SequenceOutputAction(this));
 		actionBars.setGlobalActionHandler(CommandIds.WAVE_OUTPUT, new WaveOutputAction(this));
 
@@ -1976,15 +1977,38 @@ public class LayerCompositionView extends ViewPart
 		_collectTask.schedule();
 	}
 
+
+	private static final Set<String> _connectingIds = Util.newSet();
+
 	private void connectToPlayer() {
-		MediaPlayerView playerView = UIUtil.findView(getSite().getPage(), MediaPlayerView.ID, getItemId());
-		connectToPlayer(this, playerView);
+		String itemId = getItemId();
+		if (!_connectingIds.contains(itemId)) {
+			_connectingIds.add(itemId);
+			try {
+
+				MediaPlayerView playerView = UIUtil.findView(getSite().getPage(), MediaPlayerView.ID, itemId);
+				connectToPlayer(this, playerView);
+
+			} finally {
+				_connectingIds.remove(itemId);
+			}
+		}
 	}
 
 	static void connectToPlayer(MediaPlayerView playerView) {
-		LayerCompositionView compView = UIUtil.findView(
-				playerView.getSite().getPage(), LayerCompositionView.ID, playerView.getItemId());
-		connectToPlayer(compView, playerView);
+		String itemId = playerView.getItemId();
+		if (!_connectingIds.contains(itemId)) {
+			_connectingIds.add(itemId);
+			try {
+
+				LayerCompositionView compView = UIUtil.findView(
+						playerView.getSite().getPage(), LayerCompositionView.ID, itemId);
+				connectToPlayer(compView, playerView);
+
+			} finally {
+				_connectingIds.remove(itemId);
+			}
+		}
 	}
 
 	private static void connectToPlayer(final LayerCompositionView compView, final MediaPlayerView playerView) {
@@ -2006,8 +2030,36 @@ public class LayerCompositionView extends ViewPart
 					page.removePartListener(this);
 				}
 			}
+
+			public void partActivated(IWorkbenchPart part) {
+				if (part == compView) {
+					showView(playerView, compView);
+				} else if (part == playerView) {
+					showView(compView, playerView);
+				}
+			}
+
+			private void showView(IViewPart targetView, IViewPart companionView) {
+				for (IViewPart vp : page.getViewStack(companionView)) {
+					if (vp == targetView) {
+						return;
+					}
+				}
+				for (IViewPart vp : page.getViewStack(targetView)) {
+					if (vp != targetView && page.isPartVisible(vp) &&
+							vp instanceof MediaPlayerView && ((MediaPlayerView) vp).isPinned()) {
+						return;
+					}
+				}
+				try {
+					IViewSite viewSite = targetView.getViewSite();
+					page.showView(viewSite.getId(), viewSite.getSecondaryId(), IWorkbenchPage.VIEW_VISIBLE);
+				} catch (PartInitException e) {
+					_logger.error("error activating ViewPart", e);
+				}
+			}
+
 			public void partOpened(IWorkbenchPart part) { }
-			public void partActivated(IWorkbenchPart part) { }
 			public void partDeactivated(IWorkbenchPart part) { }
 			public void partBroughtToTop(IWorkbenchPart part) { }
 		};
