@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009 Yoshikazu Kuramochi
+ * Copyright (c) 2009,2010 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -65,7 +65,6 @@ import ch.kuramo.javie.app.views.LayerCompositionView;
 import ch.kuramo.javie.core.AnimatableValue;
 import ch.kuramo.javie.core.Effect;
 import ch.kuramo.javie.core.EffectableLayer;
-import ch.kuramo.javie.core.FrameDuration;
 import ch.kuramo.javie.core.JavieRuntimeException;
 import ch.kuramo.javie.core.Keyframe;
 import ch.kuramo.javie.core.Layer;
@@ -74,6 +73,7 @@ import ch.kuramo.javie.core.LayerNature;
 import ch.kuramo.javie.core.MediaInput;
 import ch.kuramo.javie.core.MediaItemLayer;
 import ch.kuramo.javie.core.ProjectDecodeException;
+import ch.kuramo.javie.core.TimeCode;
 import ch.kuramo.javie.core.Util;
 import ch.kuramo.javie.core.services.ProjectDecoder;
 import ch.kuramo.javie.core.services.ProjectEncoder;
@@ -164,7 +164,7 @@ public class TimelineManager {
 		scrollSlider.setMinimum(0);
 		scrollSlider.setIncrement(FRAME_WIDTH);
 
-		update(Time.fromFrameNumber(0, FrameDuration.FPS_29_97), true);
+		update(Time.fromFrameNumber(0, composition.getFrameDuration()), true);
 	}
 
 	public void update(Time currentTime, boolean reveal) {
@@ -311,11 +311,13 @@ public class TimelineManager {
 			Time t = Time.fromFrameNumber(i, frameDuration);
 			int x = timeToPixels(t, frameDuration, zoom) - scroll;
 
-			String text = String.valueOf(i);
+			String text = TimeCode.toTimeCode(t, frameDuration, true, false);
 			Point extent = gc.textExtent(text);
 
 			gc.drawLine(x, extent.y, x, h);
-			gc.drawText(text, x-extent.x/2, 0);
+			if ((i/step)%3 == 0) {
+				gc.drawText(text, x-extent.x/2, 0);
+			}
 		}
 
 		int x = timeToPixels(currentTime, frameDuration, zoom) - scroll;
