@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009 Yoshikazu Kuramochi
+ * Copyright (c) 2009,2010 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -31,7 +31,7 @@ public interface PropertyDescriptor {
 
 	String getLabel();
 
-	void initialize(Effect effect);
+	void initValue(Effect effect);
 
 	void afterDecode(Project p, Effect effect) throws ProjectDecodeException;
 
