@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009 Yoshikazu Kuramochi
+ * Copyright (c) 2009,2010 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -50,6 +50,7 @@ import ch.kuramo.javie.core.MediaLayer;
 import ch.kuramo.javie.core.Project;
 import ch.kuramo.javie.core.ProjectDecodeException;
 import ch.kuramo.javie.core.RenderContext;
+import ch.kuramo.javie.core.TrackMatte;
 import ch.kuramo.javie.core.Util;
 import ch.kuramo.javie.core.VectorMediaInput;
 import ch.kuramo.javie.core.VideoBuffer;
@@ -76,21 +77,23 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 
 	private boolean _audioEnabled;
 
-	private boolean _effectsEnabled;
+	private boolean _effectsEnabled = true;
 
-	private List<Effect> _effects;
+	private List<Effect> _effects = Util.newList();
 
-	private AnimatableVec2d _depthBase;
+	private TrackMatte _trackMatte = TrackMatte.NONE;
 
-	private AnimatableString _intersectionGroup;
+	private AnimatableVec2d _depthBase = new AnimatableVec2d(new Vec2d(0, 0));
 
-	private AnimatableDouble _opacity;
+	private AnimatableString _intersectionGroup = new AnimatableString("");
 
-	private BlendMode _blendMode;
+	private AnimatableDouble _opacity = new AnimatableDouble(100d);
 
-	private AnimatableVec2d _audioLevels;
+	private BlendMode _blendMode = BlendMode.NORMAL;
 
-	private AnimatableDouble _timeRemap;
+	private AnimatableVec2d _audioLevels = new AnimatableVec2d(new Vec2d(0, 0));
+
+	private AnimatableDouble _timeRemap = new AnimatableDouble(0.0);
 
 	@Inject
 	private VideoRenderContext _vrContext;
@@ -112,16 +115,16 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 
 
 	@Override
-	protected void initialize(String name) {
-		throw new UnsupportedOperationException("Use initialize(String, boolean, boolean) method instead.");
+	public void initialize() {
+		throw new UnsupportedOperationException("Use initialize(boolean, boolean) method instead.");
 	}
 
-	protected void initialize(String name, boolean videoAvailable, boolean audioAvailable) {
+	protected void initialize(boolean videoAvailable, boolean audioAvailable) {
 		if (!videoAvailable && !audioAvailable) {
 			throw new IllegalArgumentException();
 		}
 
-		super.initialize(name);
+		super.initialize();
 
 		if (videoAvailable) {
 			_videoNature = true;
@@ -132,16 +135,6 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 			_audioNature = true;
 			_audioEnabled = true;
 		}
-
-		_effectsEnabled = true;
-		_effects = Util.newList();
-
-		_depthBase = new AnimatableVec2d(new Vec2d(0, 0));
-		_intersectionGroup = new AnimatableString("");
-		_opacity = new AnimatableDouble(100d);
-		_blendMode = BlendMode.NORMAL;
-		_audioLevels = new AnimatableVec2d(new Vec2d(0, 0));
-		_timeRemap = new AnimatableDouble(0.0);
 	}
 
 	public boolean isVideoNature() {
@@ -192,6 +185,14 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 		_effects = effects;
 	}
 
+	public TrackMatte getTrackMatte() {
+		return _trackMatte;
+	}
+
+	public void setTrackMatte(TrackMatte trackMatte) {
+		_trackMatte = trackMatte;
+	}
+
 	public AnimatableVec2d getDepthBase() {
 		return _depthBase;
 	}
@@ -262,9 +263,10 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 	public void setupVideoRenderer(
 			VideoLayerComposer composer, Camera camera, CollapseTransformation ct) {
 
-		if (!LayerNature.isVideoEnabled(this)) {
-			return;
-		}
+		// トラックマット適用時にこのチェックがあるとまずいので。同じチェックはコンポ側でもしている。
+		//if (!LayerNature.isVideoEnabled(this)) {
+		//	return;
+		//}
 
 		MediaInput input = getMediaInput();
 		if (input == null || !input.isVideoAvailable()) {
@@ -352,9 +354,9 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 
 		final double opacity;
 		if (ct == null) {
-			opacity = _opacity.value(_vrContext);
+			opacity = _opacity.value(_vrContext) / 100d;
 		} else {
-			opacity = _opacity.value(_vrContext) * ct.getOpacity() / 100d;
+			opacity = _opacity.value(_vrContext) / 100d * ct.getOpacity();
 		}
 
 		VideoLayerRenderer r = new VideoLayerRenderer() {
@@ -377,11 +379,11 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 
 					DepthBuffer depthBuffer = withDepthBuffer ? _vrSupport.createDepthBuffer(camera.getViewportSize()) : null;
 
-					_vrSupport.transform(effectedBuffer, trnsfrmdBuffer, depthBuffer, mvMatrix, prjMatrix, opacity);
+					_vrSupport.transform(effectedBuffer, trnsfrmdBuffer, depthBuffer, mvMatrix, prjMatrix);
 
 					// TODO 親コンポがコラップスしている場合、プリコンポレイヤーのエフェクトはここ？
 
-					return new VideoLayerBufferImpl(trnsfrmdBuffer, depthBuffer, _blendMode);
+					return new VideoLayerBufferImpl(trnsfrmdBuffer, depthBuffer, _blendMode, opacity);
 
 				} finally {
 					if (effectedBuffer != null) {
@@ -425,9 +427,9 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 
 		final double opacity;
 		if (ct == null) {
-			opacity = _opacity.value(_vrContext);
+			opacity = _opacity.value(_vrContext) / 100d;
 		} else {
-			opacity = _opacity.value(_vrContext) * ct.getOpacity() / 100d;
+			opacity = _opacity.value(_vrContext) / 100d * ct.getOpacity();
 		}
 
 		VideoLayerRenderer r = new VideoLayerRenderer() {
@@ -442,7 +444,7 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 				_vrContext.saveAndExecute(new WrappedOperation<Object>() {
 					public Object execute() {
 						setMediaTime(_vrContext, input);
-						input.rasterize(rasterBuffer, depthBuffer, mvMatrix, prjMatrix, opacity);
+						input.rasterize(rasterBuffer, depthBuffer, mvMatrix, prjMatrix);
 						return null;
 					}
 				});
@@ -450,7 +452,7 @@ public abstract class AbstractMediaLayer extends AbstractTransformableLayer impl
 				// TODO このレイヤーのエフェクト
 				// TODO 親コンポがコラップスしている場合、プリコンポレイヤーのエフェクトはここ？
 
-				return new VideoLayerBufferImpl(rasterBuffer, depthBuffer, _blendMode);
+				return new VideoLayerBufferImpl(rasterBuffer, depthBuffer, _blendMode, opacity);
 			}
 		};
 
