@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009 Yoshikazu Kuramochi
+ * Copyright (c) 2009,2010 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -29,6 +29,7 @@ import java.util.ListIterator;
 import ch.kuramo.javie.api.ColorMode;
 import ch.kuramo.javie.api.Size2i;
 import ch.kuramo.javie.api.Time;
+import ch.kuramo.javie.api.Vec3d;
 import ch.kuramo.javie.core.AbstractComposition;
 import ch.kuramo.javie.core.AudioBuffer;
 import ch.kuramo.javie.core.Camera;
@@ -42,9 +43,12 @@ import ch.kuramo.javie.core.MediaLayer;
 import ch.kuramo.javie.core.Project;
 import ch.kuramo.javie.core.ProjectDecodeException;
 import ch.kuramo.javie.core.RenderContext;
+import ch.kuramo.javie.core.TrackMatte;
 import ch.kuramo.javie.core.Util;
 import ch.kuramo.javie.core.VideoBuffer;
+import ch.kuramo.javie.core.VideoLayerBuffer;
 import ch.kuramo.javie.core.VideoLayerComposer;
+import ch.kuramo.javie.core.VideoLayerRenderer;
 import ch.kuramo.javie.core.WrappedOperation;
 import ch.kuramo.javie.core.annotations.ProjectElement;
 import ch.kuramo.javie.core.services.AudioRenderContext;
@@ -136,7 +140,9 @@ public class LayerCompositionImpl extends AbstractComposition implements LayerCo
 		final VideoLayerComposerImpl composer = new VideoLayerComposerImpl(
 				_vrContext, _vrSupport, colorMode, size, camera.getPosition());
 
-		for (Layer layer : _layers) {
+		for (int i = 0, n = _layers.size(); i < n; ++i) {
+			Layer layer = _layers.get(i);
+
 			if (!LayerNature.isVideoEnabled(layer)) {
 				continue;
 			}
@@ -147,6 +153,15 @@ public class LayerCompositionImpl extends AbstractComposition implements LayerCo
 
 			if (layer instanceof MediaLayer) {
 				final MediaLayer ml = (MediaLayer) layer;
+
+				if (ml.getTrackMatte() != TrackMatte.NONE && i+1 < n) {
+					Layer matteLayer = _layers.get(i+1);
+					if (matteLayer instanceof MediaLayer) {
+						setupTrackMatteRenderer(ml, (MediaLayer) matteLayer, composer, camera, null);
+						continue;
+					}
+				}
+
 				_vrContext.saveAndExecute(new WrappedOperation<Object>() {
 					public Object execute() {
 						ml.setupVideoRenderer(composer, camera, null);
@@ -162,13 +177,30 @@ public class LayerCompositionImpl extends AbstractComposition implements LayerCo
 	public void setupCollapseTransformation(
 			final VideoLayerComposer composer, final Camera camera, final CollapseTransformation ct) {
 
-		for (Layer layer : _layers) {
+		Time time = _vrContext.getTime();
+
+		for (int i = 0, n = _layers.size(); i < n; ++i) {
+			Layer layer = _layers.get(i);
+
 			if (!LayerNature.isVideoEnabled(layer)) {
 				continue;
 			}
 
+			if (layer.getInPoint().after(time) || !layer.getOutPoint().after(time)) {
+				continue;
+			}
+
 			if (layer instanceof MediaLayer) {
 				final MediaLayer ml = (MediaLayer) layer;
+
+				if (ml.getTrackMatte() != TrackMatte.NONE && i+1 < n) {
+					Layer matteLayer = _layers.get(i+1);
+					if (matteLayer instanceof MediaLayer) {
+						setupTrackMatteRenderer(ml, (MediaLayer) matteLayer, composer, camera, ct);
+						continue;
+					}
+				}
+
 				_vrContext.saveAndExecute(new WrappedOperation<Object>() {
 					public Object execute() {
 						ml.setupVideoRenderer(composer, camera, ct);
@@ -179,6 +211,82 @@ public class LayerCompositionImpl extends AbstractComposition implements LayerCo
 		}
 	}
 
+	private void setupTrackMatteRenderer(
+			MediaLayer fillLayer, MediaLayer matteLayer,
+			VideoLayerComposer composer, Camera camera, CollapseTransformation ct) {
+
+		final VideoLayerRenderer[] renderers = new VideoLayerRenderer[2];
+		final Vec3d[][] vertices = new Vec3d[1][];
+		final String[] intersectionGroup = new String[1];
+
+		fillLayer.setupVideoRenderer(new VideoLayerComposer() {
+			public void add2D(VideoLayerRenderer r)							{ renderers[0] = r; }
+			public void add3D(VideoLayerRenderer r, Vec3d[] v, String ig)	{ renderers[0] = r; vertices[0] = v; intersectionGroup[0] = ig; }
+		}, camera, ct);
+
+		matteLayer.setupVideoRenderer(new VideoLayerComposer() {
+			public void add2D(VideoLayerRenderer r)							{ renderers[1] = r; }
+			public void add3D(VideoLayerRenderer r, Vec3d[] v, String ig)	{ renderers[1] = r; }
+		}, camera, ct);
+
+
+		if (renderers[0] == null) {
+			return;
+
+		} else if (renderers[1] == null) {
+			if (vertices[0] == null) {
+				composer.add2D(renderers[0]);
+			} else {
+				composer.add3D(renderers[0], vertices[0], intersectionGroup[0]);
+			}
+			return;
+		}
+
+
+		final TrackMatte trackMatte = fillLayer.getTrackMatte();
+
+		VideoLayerRenderer r = new VideoLayerRenderer() {
+			public VideoLayerBuffer render(boolean withDepthBuffer) {
+				VideoLayerBuffer fillBuffer = null;
+				VideoLayerBuffer matteBuffer = null;
+				VideoBuffer vbFill = null;
+				try {
+					fillBuffer = renderers[0].render(withDepthBuffer);
+					matteBuffer = renderers[1].render(false);
+					vbFill = fillBuffer.getVideoBuffer();
+
+					VideoBuffer resultBuffer = _vrSupport.createVideoBuffer(vbFill.getColorMode(), vbFill.getBounds());
+					_vrSupport.trackMatte(matteBuffer.getVideoBuffer(), fillBuffer.getVideoBuffer(), resultBuffer,
+							trackMatte, matteBuffer.getOpacity());
+
+					VideoLayerBuffer newBuffer = new VideoLayerBufferImpl(
+							resultBuffer, fillBuffer.getDepthBuffer(), fillBuffer.getBlendMode(), fillBuffer.getOpacity());
+					fillBuffer = null;
+					return newBuffer;
+
+				} finally {
+					if (matteBuffer != null) {
+						matteBuffer.dispose();
+					}
+
+					// 正常時はfillBufferを破棄してしまうとDepthBufferまで破棄されてしまうので、VideoBufferだけ破棄する。
+					// (tryブロックを抜ける直前にfillBufferにnullを代入しているので、ここでfillBufferがnullでないのは例外発生時のみ)
+					if (fillBuffer != null) {
+						fillBuffer.dispose();
+					} else if (vbFill != null) {
+						vbFill.dispose();
+					}
+				}
+			}
+		};
+
+		if (vertices[0] == null) {
+			composer.add2D(r);
+		} else {
+			composer.add3D(r, vertices[0], intersectionGroup[0]);
+		}
+	}
+
 	public AudioBuffer renderAudioChunk() {
 		AudioBuffer ab = null;
 		Time time = _arContext.getTime();
