@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009 Yoshikazu Kuramochi
+ * Copyright (c) 2009,2010 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -57,7 +57,7 @@ public class MediaItemLayerImpl extends AbstractMediaLayer implements MediaItemL
 
 
 	@Override
-	protected void initialize(String name, boolean videoAvailable, boolean audioAvailable) {
+	protected void initialize(boolean videoAvailable, boolean audioAvailable) {
 		throw new UnsupportedOperationException("Use initialize(MediaItem) method instead.");
 	}
 
@@ -70,10 +70,12 @@ public class MediaItemLayerImpl extends AbstractMediaLayer implements MediaItemL
 			throw new IllegalArgumentException("No video nor audio is available.");
 		}
 
-		super.initialize(mediaItem.getName(), videoAvailable, audioAvailable);
+		super.initialize(videoAvailable, audioAvailable);
 
 		_itemId = mediaItem.getId();
 		_mediaItem = mediaItem;
+
+		setName(mediaItem.getName());
 	}
 
 	public String getItemId() {
@@ -159,9 +161,9 @@ public class MediaItemLayerImpl extends AbstractMediaLayer implements MediaItemL
 
 		final double opacity;
 		if (ct != null) {
-			opacity = getOpacity().value(_vrContext) * ct.getOpacity();
+			opacity = getOpacity().value(_vrContext) / 100d * ct.getOpacity();
 		} else {
-			opacity = getOpacity().value(_vrContext);
+			opacity = getOpacity().value(_vrContext) / 100d;
 		}
 
 		final CollapseTransformation ct2 = new CollapseTransformation() {
