@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009 Yoshikazu Kuramochi
+ * Copyright (c) 2009,2010 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -147,12 +147,11 @@ public class SolidColorItemImpl extends AbstractItem implements SolidColorItem {
 
 		public void rasterize(
 				VideoBuffer resultBuffer, DepthBuffer depthBuffer,
-				double[] mvMatrix, double[] prjMatrix, double opacity) {
+				double[] mvMatrix, double[] prjMatrix) {
 
 			Size2i size = _vrContext.getRenderResolution().scale(_size);
 
-			_vrSupport.fillRectCR(resultBuffer, depthBuffer,
-					mvMatrix, prjMatrix, opacity, size, _color);
+			_vrSupport.fillRectCR(resultBuffer, depthBuffer, mvMatrix, prjMatrix, size, _color);
 		}
 
 		public AudioBuffer getAudioChunk() {
