@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009 Yoshikazu Kuramochi
+ * Copyright (c) 2009,2010 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -289,7 +289,7 @@ class EffectDescriptorImpl implements EffectDescriptor {
 			this.value = toDouble(value);
 		}
 
-		public void initialize(Effect effect) {
+		public void initValue(Effect effect) {
 			set(effect, new AnimatableDouble(value));
 		}
 	}
@@ -303,7 +303,7 @@ class EffectDescriptorImpl implements EffectDescriptor {
 			this.value = toInteger(value);
 		}
 
-		public void initialize(Effect effect) {
+		public void initValue(Effect effect) {
 			set(effect, new AnimatableInteger(value));
 		}
 	}
@@ -317,7 +317,7 @@ class EffectDescriptorImpl implements EffectDescriptor {
 			this.value = toVec2d(value);
 		}
 
-		public void initialize(Effect effect) {
+		public void initValue(Effect effect) {
 			set(effect, new AnimatableVec2d(value));
 		}
 	}
@@ -331,7 +331,7 @@ class EffectDescriptorImpl implements EffectDescriptor {
 			this.value = toVec3d(value);
 		}
 
-		public void initialize(Effect effect) {
+		public void initValue(Effect effect) {
 			set(effect, new AnimatableVec3d(value));
 		}
 	}
@@ -345,7 +345,7 @@ class EffectDescriptorImpl implements EffectDescriptor {
 			this.value = toColor(value);
 		}
 
-		public void initialize(Effect effect) {
+		public void initValue(Effect effect) {
 			set(effect, new AnimatableColor(value));
 		}
 	}
@@ -359,7 +359,7 @@ class EffectDescriptorImpl implements EffectDescriptor {
 			this.value = value;
 		}
 
-		public void initialize(Effect effect) {
+		public void initValue(Effect effect) {
 			set(effect, new AnimatableString(value));
 		}
 	}
@@ -373,7 +373,7 @@ class EffectDescriptorImpl implements EffectDescriptor {
 			this.value = toBoolean(value);
 		}
 
-		public void initialize(Effect effect) {
+		public void initValue(Effect effect) {
 			set(effect, new AnimatableBoolean(value));
 		}
 	}
@@ -399,7 +399,7 @@ class EffectDescriptorImpl implements EffectDescriptor {
 			}
 		}
 
-		public void initialize(Effect effect) {
+		public void initValue(Effect effect) {
 			try {
 				set(effect, constructor.newInstance(value));
 			} catch (InstantiationException e) {
@@ -421,7 +421,7 @@ class EffectDescriptorImpl implements EffectDescriptor {
 			this.value = value;
 		}
 
-		public void initialize(Effect effect) {
+		public void initValue(Effect effect) {
 			set(effect, value);
 		}
 	}
@@ -436,7 +436,7 @@ class EffectDescriptorImpl implements EffectDescriptor {
 			this.value = toEnum(enumType, value);
 		}
 
-		public void initialize(Effect effect) {
+		public void initValue(Effect effect) {
 			set(effect, value);
 		}
 	}
@@ -451,7 +451,7 @@ class EffectDescriptorImpl implements EffectDescriptor {
 			this.value = toPrimitive(primitiveType, value);
 		}
 
-		public void initialize(Effect effect) {
+		public void initValue(Effect effect) {
 			set(effect, value);
 		}
 	}
