@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009 Yoshikazu Kuramochi
+ * Copyright (c) 2009,2010 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -43,6 +43,7 @@ import ch.kuramo.javie.api.Vec3d;
 import ch.kuramo.javie.api.VideoBounds;
 import ch.kuramo.javie.core.BlendMode;
 import ch.kuramo.javie.core.DepthBuffer;
+import ch.kuramo.javie.core.TrackMatte;
 import ch.kuramo.javie.core.VideoBuffer;
 import ch.kuramo.javie.core.VideoLayerBuffer;
 import ch.kuramo.javie.core.WrappedOperation;
@@ -55,6 +56,7 @@ import ch.kuramo.javie.core.services.ShaderRegistry;
 import ch.kuramo.javie.core.services.VideoRenderContext;
 import ch.kuramo.javie.core.services.VideoRenderSupport;
 import ch.kuramo.javie.core.shaders.BlendModeShaders;
+import ch.kuramo.javie.core.shaders.TrackMatteShaders;
 
 import com.google.inject.Inject;
 import com.google.inject.Injector;
@@ -336,7 +338,7 @@ public class VideoRenderSupportImpl implements VideoRenderSupport {
 
 	public void fillRectCR(
 			VideoBuffer resultBuffer, DepthBuffer depthBuffer,
-			final double[] mvMatrix, final double[] prjMatrix, final double opacity,
+			final double[] mvMatrix, final double[] prjMatrix,
 			final Size2i size, final Color color) {
 
 		final int resultTexture = resultBuffer.getTexture();
@@ -368,21 +370,24 @@ public class VideoRenderSupportImpl implements VideoRenderSupport {
 					gl.glClear(GL.GL_DEPTH_BUFFER_BIT);
 				}
 
-				gl.glBegin(GL.GL_QUADS);
+				float[] savedColor = new float[4];
+				gl.glGetFloatv(GL.GL_CURRENT_COLOR, savedColor, 0);
 
-				float a = (float)(color.a * opacity / 100.0);
+				float a = (float)color.a;
 				float r = (float)color.r * a;
 				float g = (float)color.g * a;
 				float b = (float)color.b * a;
 				gl.glColor4f(r, g, b, a);
 
+				gl.glBegin(GL.GL_QUADS);
 				gl.glVertex2i(0, 0);
 				gl.glVertex2i(size.width, 0);
 				gl.glVertex2i(size.width, size.height);
 				gl.glVertex2i(0, size.height);
-
 				gl.glEnd();
 
+				gl.glColor4fv(savedColor, 0);
+
 				if (depthTexture != 0) {
 					gl.glDisable(GL.GL_DEPTH_TEST);
 					gl.glFramebufferTexture2DEXT(GL.GL_FRAMEBUFFER_EXT,
@@ -412,7 +417,73 @@ public class VideoRenderSupportImpl implements VideoRenderSupport {
 		gl.glLoadIdentity();
 	}
 
-	public void blend(VideoBuffer srcBuffer, VideoBuffer dstBuffer, VideoBuffer resultBuffer, final BlendMode blendMode) {
+	public void trackMatte(
+			VideoBuffer matteBuffer, VideoBuffer fillBuffer, VideoBuffer resultBuffer,
+			final TrackMatte trackMatte, final double matteOpacity) {
+
+		if (trackMatte == TrackMatte.NONE) {
+			throw new IllegalArgumentException();
+		}
+
+		final int matteTexture = matteBuffer.getTexture();
+		final int fillTexture = fillBuffer.getTexture();
+		final int resultTexture = resultBuffer.getTexture();
+		final Size2i imageSize = resultBuffer.getImageSize();
+
+		pushMatrixAndExecute(new WrappedOperation<Object>() {
+			public Object execute() {
+				GL gl = _vrContext.getGL();
+				GLU glu = _vrContext.getGLU();
+
+				ortho2D(gl, glu, imageSize);
+
+				// レンダリング先を resultTexture に設定
+				gl.glFramebufferTexture2DEXT(GL.GL_FRAMEBUFFER_EXT,
+						GL.GL_COLOR_ATTACHMENT0_EXT, GL.GL_TEXTURE_RECTANGLE_EXT, resultTexture, 0);
+				gl.glDrawBuffer(GL.GL_COLOR_ATTACHMENT0_EXT);
+
+				// レンダリングのソースとなるテクスチャユニットの設定
+				gl.glActiveTexture(GL.GL_TEXTURE0);
+				gl.glBindTexture(GL.GL_TEXTURE_RECTANGLE_EXT, fillTexture);
+				gl.glActiveTexture(GL.GL_TEXTURE1);
+				gl.glBindTexture(GL.GL_TEXTURE_RECTANGLE_EXT, matteTexture);
+
+				ShaderProgram program = _shaders.getProgram(TrackMatteShaders.class, trackMatte.name());
+				synchronized (program) {
+					gl.glUseProgram(program.getProgram());
+					gl.glUniform1i(program.getUniformLocation("texFill"), 0);
+					gl.glUniform1i(program.getUniformLocation("texMatte"), 1);
+					gl.glUniform1f(program.getUniformLocation("matteOpacity"), (float)matteOpacity);
+
+					gl.glBegin(GL.GL_QUADS);
+					gl.glTexCoord2f(0, 0);
+					gl.glVertex2f(0, 0);
+					gl.glTexCoord2f(imageSize.width, 0);
+					gl.glVertex2f(imageSize.width, 0);
+					gl.glTexCoord2f(imageSize.width, imageSize.height);
+					gl.glVertex2f(imageSize.width, imageSize.height);
+					gl.glTexCoord2f(0, imageSize.height);
+					gl.glVertex2f(0, imageSize.height);
+					gl.glEnd();
+
+					gl.glFinish();
+					gl.glUseProgram(0);
+				}
+
+				gl.glActiveTexture(GL.GL_TEXTURE1);
+				gl.glBindTexture(GL.GL_TEXTURE_RECTANGLE_EXT, 0);
+				gl.glActiveTexture(GL.GL_TEXTURE0);
+				gl.glBindTexture(GL.GL_TEXTURE_RECTANGLE_EXT, 0);
+
+				gl.glFramebufferTexture2DEXT(GL.GL_FRAMEBUFFER_EXT,
+						GL.GL_COLOR_ATTACHMENT0_EXT, GL.GL_TEXTURE_RECTANGLE_EXT, 0, 0);
+
+				return null;
+			}
+		});
+	}
+
+	public void blend(VideoBuffer srcBuffer, VideoBuffer dstBuffer, VideoBuffer resultBuffer, final BlendMode blendMode, final double opacity) {
 
 		final int srcTexture = srcBuffer.getTexture();
 		final int dstTexture = dstBuffer.getTexture();
@@ -433,15 +504,19 @@ public class VideoRenderSupportImpl implements VideoRenderSupport {
 
 				// レンダリングのソースとなるテクスチャユニットの設定
 				gl.glActiveTexture(GL.GL_TEXTURE0);
-				gl.glBindTexture(GL.GL_TEXTURE_RECTANGLE_EXT, srcTexture);
-				gl.glActiveTexture(GL.GL_TEXTURE1);
 				gl.glBindTexture(GL.GL_TEXTURE_RECTANGLE_EXT, dstTexture);
+				gl.glActiveTexture(GL.GL_TEXTURE1);
+				gl.glBindTexture(GL.GL_TEXTURE_RECTANGLE_EXT, srcTexture);
 
 				ShaderProgram program = _shaders.getProgram(BlendModeShaders.class, blendMode.name());
 				synchronized (program) {
 					gl.glUseProgram(program.getProgram());
-					gl.glUniform1i(program.getUniformLocation("texSrc"), 0);
-					gl.glUniform1i(program.getUniformLocation("texDst"), 1);
+					gl.glUniform1i(program.getUniformLocation("texDst"), 0);
+					gl.glUniform1i(program.getUniformLocation("texSrc"), 1);
+					gl.glUniform1f(program.getUniformLocation("opacity"), (float)opacity);
+					if (blendMode == BlendMode.DANCING_DISSOLVE) {
+						gl.glUniform1f(program.getUniformLocation("time"), (float)(_vrContext.getTime().toSecond()));
+					}
 
 					gl.glBegin(GL.GL_QUADS);
 					gl.glTexCoord2f(0, 0);
@@ -454,7 +529,7 @@ public class VideoRenderSupportImpl implements VideoRenderSupport {
 					gl.glVertex2f(0, imageSize.height);
 					gl.glEnd();
 
-					gl.glFlush();
+					gl.glFinish();
 					gl.glUseProgram(0);
 				}
 
@@ -496,6 +571,7 @@ public class VideoRenderSupportImpl implements VideoRenderSupport {
 				}
 
 				int[] blendMode = new int[maxLayers];
+				float[] opacity = new float[maxLayers];
 
 				// レンダリング先を resultTexture に設定
 				gl.glFramebufferTexture2DEXT(GL.GL_FRAMEBUFFER_EXT,
@@ -513,6 +589,7 @@ public class VideoRenderSupportImpl implements VideoRenderSupport {
 					gl.glActiveTexture(GL.GL_TEXTURE0 + i + 1 + maxLayers);
 					gl.glBindTexture(GL.GL_TEXTURE_RECTANGLE_EXT, vlb.getDepthBuffer().getTexture());
 					blendMode[i] = vlb.getBlendMode().ordinal();
+					opacity[i] = (float)vlb.getOpacity();
 					++i;
 					if (i >= nLayers) break;
 				}
@@ -527,6 +604,8 @@ public class VideoRenderSupportImpl implements VideoRenderSupport {
 						gl.glUniform1i(program.getUniformLocation("texDep" + i), i + 1 + maxLayers);
 					}
 					gl.glUniform1iv(program.getUniformLocation("blendMode[0]"), maxLayers, blendMode, 0);
+					gl.glUniform1fv(program.getUniformLocation("opacity[0]"), maxLayers, opacity, 0);
+					gl.glUniform1f(program.getUniformLocation("time"), (float)(_vrContext.getTime().toSecond()));
 
 					gl.glBegin(GL.GL_QUADS);
 					gl.glTexCoord2f(0, 0);
@@ -539,7 +618,7 @@ public class VideoRenderSupportImpl implements VideoRenderSupport {
 					gl.glVertex2f(0, imageSize.height);
 					gl.glEnd();
 
-					gl.glFlush();
+					gl.glFinish();
 					gl.glUseProgram(0);
 				}
 
@@ -556,6 +635,39 @@ public class VideoRenderSupportImpl implements VideoRenderSupport {
 		});
 	}
 
+	public void premultiply(final VideoBuffer vb) {
+		pushMatrixAndExecute(new WrappedOperation<Object>() {
+			public Object execute() {
+				GL gl = _vrContext.getGL();
+				GLU glu = _vrContext.getGLU();
+
+				VideoBounds bounds = vb.getBounds();
+				ortho2D(gl, glu, bounds.width, bounds.height);
+
+				gl.glFramebufferTexture2DEXT(GL.GL_FRAMEBUFFER_EXT,
+						GL.GL_COLOR_ATTACHMENT0_EXT, GL.GL_TEXTURE_RECTANGLE_EXT, vb.getTexture(), 0);
+				gl.glDrawBuffer(GL.GL_COLOR_ATTACHMENT0_EXT);
+
+				gl.glEnable(GL.GL_BLEND);
+				gl.glBlendFuncSeparate(GL.GL_ZERO, GL.GL_DST_ALPHA, GL.GL_ZERO, GL.GL_ONE);
+
+				gl.glBegin(GL.GL_QUADS);
+				gl.glVertex2f(0, 0);
+				gl.glVertex2f(bounds.width, 0);
+				gl.glVertex2f(bounds.width, bounds.height);
+				gl.glVertex2f(0, bounds.height);
+				gl.glEnd();
+
+				gl.glDisable(GL.GL_BLEND);
+
+				gl.glFramebufferTexture2DEXT(GL.GL_FRAMEBUFFER_EXT,
+						GL.GL_COLOR_ATTACHMENT0_EXT, GL.GL_TEXTURE_RECTANGLE_EXT, 0, 0);
+
+				return null;
+			}
+		});
+	}
+
 	public <T> T pushMatrixAndExecute(WrappedOperation<T> operation) {
 		GL gl = _vrContext.getGL();
 
@@ -750,7 +862,7 @@ public class VideoRenderSupportImpl implements VideoRenderSupport {
 
 	public void transform(
 			VideoBuffer srcBuffer, VideoBuffer resultBuffer, DepthBuffer depthBuffer,
-			final double[] mvMatrix, final double[] prjMatrix, final double opacity) {
+			final double[] mvMatrix, final double[] prjMatrix) {
 
 		final int srcTexture = srcBuffer.getTexture();
 		final Size2i srcImageSize = srcBuffer.getImageSize();
@@ -787,25 +899,24 @@ public class VideoRenderSupportImpl implements VideoRenderSupport {
 					gl.glClear(GL.GL_DEPTH_BUFFER_BIT);
 				}
 
+				float[] savedColor = new float[4];
+				gl.glGetFloatv(GL.GL_CURRENT_COLOR, savedColor, 0);
+
+				gl.glColor4f(1, 1, 1, 1);
+
 				gl.glBegin(GL.GL_QUADS);
-
-				float a = (float)(opacity / 100.0);
-				gl.glColor4f(a, a, a, a);
-
 				gl.glTexCoord2f(0, 0);
 				gl.glVertex2f(0, 0);
-
 				gl.glTexCoord2f(srcImageSize.width, 0);
 				gl.glVertex2f(srcImageSize.width, 0);
-
 				gl.glTexCoord2f(srcImageSize.width, srcImageSize.height);
 				gl.glVertex2f(srcImageSize.width, srcImageSize.height);
-
 				gl.glTexCoord2f(0, srcImageSize.height);
 				gl.glVertex2f(0, srcImageSize.height);
-
 				gl.glEnd();
 
+				gl.glColor4fv(savedColor, 0);
+
 				if (depthTexture != 0) {
 					gl.glDisable(GL.GL_DEPTH_TEST);
 					gl.glFramebufferTexture2DEXT(GL.GL_FRAMEBUFFER_EXT,
@@ -851,23 +962,24 @@ public class VideoRenderSupportImpl implements VideoRenderSupport {
 				gl.glBindTexture(GL.GL_TEXTURE_RECTANGLE_EXT, srcTexture);
 				gl.glEnable(GL.GL_TEXTURE_RECTANGLE_EXT);
 
-				gl.glBegin(GL.GL_QUADS);
+				float[] savedColor = new float[4];
+				gl.glGetFloatv(GL.GL_CURRENT_COLOR, savedColor, 0);
+
 				gl.glColor4f(1, 1, 1, 1);
 
+				gl.glBegin(GL.GL_QUADS);
 				gl.glTexCoord2f(0, 0);
 				gl.glVertex2f(0, 0);
-
 				gl.glTexCoord2f(srcImageSize.width, 0);
 				gl.glVertex2f(srcImageSize.width, 0);
-
 				gl.glTexCoord2f(srcImageSize.width, srcImageSize.height);
 				gl.glVertex2f(srcImageSize.width, srcImageSize.height);
-
 				gl.glTexCoord2f(0, srcImageSize.height);
 				gl.glVertex2f(0, srcImageSize.height);
-
 				gl.glEnd();
 
+				gl.glColor4fv(savedColor, 0);
+
 				gl.glBindTexture(GL.GL_TEXTURE_RECTANGLE_EXT, 0);
 				gl.glDisable(GL.GL_TEXTURE_RECTANGLE_EXT);
 				gl.glFramebufferTexture2DEXT(GL.GL_FRAMEBUFFER_EXT,
@@ -900,11 +1012,10 @@ public class VideoRenderSupportImpl implements VideoRenderSupport {
 				// これをしておかないと ftglRenderFont 内の glEnable(GL_TEXTURE_2D) でエラー1282になる。
 				gl.glActiveTexture(GL.GL_TEXTURE0);
 
-				float a = (float)fillColor.a;
-				float r = (float)fillColor.r * a; 
-				float g = (float)fillColor.g * a; 
-				float b = (float)fillColor.b * a; 
-				gl.glColor4f(r, g, b, a);
+				float[] savedColor = new float[4];
+				gl.glGetFloatv(GL.GL_CURRENT_COLOR, savedColor, 0);
+
+				gl.glColor4f((float)fillColor.r, (float)fillColor.g, (float)fillColor.b, (float)fillColor.a);
 
 				// オリジナルのFTGLは内部でアルファブレンドの設定を行っているが、Javieはそれを無効にした修正版FTGLを使用している。
 				// Javieには次のアルファブレンドが必要なため。
@@ -915,6 +1026,8 @@ public class VideoRenderSupportImpl implements VideoRenderSupport {
 
 				gl.glDisable(GL.GL_BLEND);
 
+				gl.glColor4fv(savedColor, 0);
+
 				gl.glFramebufferTexture2DEXT(GL.GL_FRAMEBUFFER_EXT,
 						GL.GL_COLOR_ATTACHMENT0_EXT, GL.GL_TEXTURE_RECTANGLE_EXT, 0, 0);
 
@@ -926,7 +1039,7 @@ public class VideoRenderSupportImpl implements VideoRenderSupport {
 	public void renderText(
 			final FTGLfont font, final Color fillColor, final String text,
 			VideoBuffer resultBuffer, DepthBuffer depthBuffer,
-			final double[] mvMatrix, final double[] prjMatrix, final double opacity) {
+			final double[] mvMatrix, final double[] prjMatrix) {
 
 		final int resultTexture = resultBuffer.getTexture();
 		final Size2i resultImageSize = resultBuffer.getImageSize();
@@ -963,7 +1076,10 @@ public class VideoRenderSupportImpl implements VideoRenderSupport {
 					gl.glClear(GL.GL_DEPTH_BUFFER_BIT);
 				}
 
-				float a = (float)(fillColor.a * opacity / 100.0);
+				float[] savedColor = new float[4];
+				gl.glGetFloatv(GL.GL_CURRENT_COLOR, savedColor, 0);
+
+				float a = (float)fillColor.a;
 				float r = (float)fillColor.r * a;
 				float g = (float)fillColor.g * a;
 				float b = (float)fillColor.b * a;
@@ -977,6 +1093,8 @@ public class VideoRenderSupportImpl implements VideoRenderSupport {
 							GL.GL_DEPTH_ATTACHMENT_EXT, GL.GL_TEXTURE_RECTANGLE_EXT, 0, 0);
 				}
 
+				gl.glColor4fv(savedColor, 0);
+
 				gl.glFramebufferTexture2DEXT(GL.GL_FRAMEBUFFER_EXT,
 						GL.GL_COLOR_ATTACHMENT0_EXT, GL.GL_TEXTURE_RECTANGLE_EXT, 0, 0);
 
@@ -985,8 +1103,4 @@ public class VideoRenderSupportImpl implements VideoRenderSupport {
 		});
 	}
 
-	public void flush() {
-		_vrContext.getGL().glFlush();
-	}
-
 }
